#include "main.h"
#include "hal.h"

#include "globals.h"
#include "xypt.h"

volatile u16 servo_value=100;
volatile u32 timeout_50hz=0;

ISR(TIMER0_OVF_vect)
{
  static u8 num=0;

  if(num==0)
  {
    digitalOut(13,1);
    OCR0A = (u8)servo_value;
    TIMSK0 = (1<<TOIE0) | (1<<OCIE0A);
    TIFR0 = (1<<OCF0A);

    if ( timeout_50hz > 0 )
      timeout_50hz --;
  }
  if(num==5)
    num=0;
  else
    num++;
}

ISR(TIMER0_COMPA_vect)
{
  digitalOut(13,0);
  TIMSK0 = (1<<TOIE0);
}

void tim0_set()
{
  // 16MHz / 256 = 16us
  //2000us/16us=125
  //prescaler=256
  //limit=
  //TCCR0A: WGM00 WGM01 COM0B0 COM0B1 COM0A0 COM0A1
  //TCCR0B: FOC0A FOC0B WGM02 CS02 CS01 CS00
  //TCNT0: counter
  //OCR0A, OCR0B: cmp
  //TIMSK: OCIE0B OCIE0A TOIE0
  // normal mode, WGM02:0=0
  // TOV0 -> overflow
  // CS02:0 -> clock select = 100 (clk/256)
  TCCR0B = (1<<CS02);
  TIMSK0 = (1<<TOIE0);
}

void setup()
{
  //
  uart(BAUD);

  //
  pinMode(13, OUT);
  //hal.timer2_init(1000); // 500=500ms, for clock, in ms!
  //hal.timer2_start();

  //for(u8 i=2;i<=9;i++)
  for(u8 i=2;i<=5;i++) //stepmotors
    pinMode(i, OUT);

  //hal.timer1_init(500000l); // 5e5=5e5us(1/2s), in us!
  //hal.timer1_start();
  //
  tim0_set();
  sei();
  
  //debug_init();
  //hal.uart_puts("START\r\n");
  //hal.uart_puts("and next START\r\n");
  pinMode(6,IN); //R endstop
  pinMode(7,IN); //L endstop
  pinMode(8,IN); //xx in
}

extern s32 motor1_curr;
extern s32 motor2_curr;
extern s32 motor1_need;
extern s32 motor2_need;
extern u8 queue_is_full;

void do_steps()
{
  if( (motor1_curr == motor1_need) 
   && (motor2_curr == motor2_need) 
   && (timeout_50hz==0))
  {
    if(queue_status() != QUEUE_EMPTY)
    {
      XYPT xypt;
      bool ret;
      cli();
      ret = queue_get(&xypt);
      sei();
    //hal.uart_puts("G1\r\n");
      if(ret)
      {
        //do next step x,y,p,t
        motor1_need = xypt.x;
        motor2_need = xypt.y;
        servo_value = xypt.p;
        //xypt.t
        //hal.timer2_init(1000); // 500=500ms, for clock, in ms!
        //hal.timer2_start();
        //hal.timer1_init(500000l); // 5e5=5e5us(1/2s), in us!
        //hal.timer1_start();

        timer1_off_hz();
        timer2_off_hz();
        timer1_on_hz((u32)abs((xypt.x*1000L)/xypt.t));
        timer2_on_hz((u32)abs((xypt.y*1000L)/xypt.t));
        timeout_50hz = (xypt.t+19)/20; // on 20ms intervals
        //hal.timer1_start();
        //hal.timer2_start();
      }
    }
  } // end of 'need new steps'

  if( (queue_is_full==1)
    &&(queue_status()<=QUEUE_LT_SUB_EMPTY))
  {
    queue_is_full=0;
    uart_puts("OK2\r\n");
  }
}

void loop()
{
  do_steps();
}

//  hal.digitalOut(13,1);
//  hal.pinMode(13,OUT);

void timer1_callback()
{
  update_motor1();
  /*
  static u8 a=0;
  a=1-a;
  hal.digitalOut(13,a);
  */
}

void timer2_callback()
{
  update_motor2();
  /*
  static u8 a=0;
  a=1-a;
  hal.digitalOut(13,a);
  */
}
